        subroutine alfcalc()
            use vars
            Implicit Double Precision (A-H,O-Z)
            double precision :: sinrho

            sinrho = (H0*vs(7))/(1d0+H0*vs(7)/sin(phirad))
            alpha = (2d0*sinrho)/((sqrt(3d0))*(3d0-sinrho)) ! Matching TXC
            !alpha = (2d0*sinrho)/((sqrt(3d0))*(3d0+sinrho)) ! Matching TXE
            !alpha = sinrho/((sqrt(3d0))*(sqrt(3d0+sinrho**2))) ! Tangential inner circle
            xky = 3d0*alpha*a

        end subroutine

        subroutine Dmatrix (D)
            use vars
            Implicit Double Precision (A-H,O-Z)
            Real(8) :: D(6,6), e,p0

            p0 = (v0(1)+v0(2)+v0(3))/3d0
            if (p0.lt.1d0) then
                p0 = 1d0
            end if
            xG = c(1) * sqrt(p0/c(10))
            BFac = (2d0*(1d0+c(2)))/(3d0*(1-2d0*c(2))) ! K/G as function of Nu (Nu=c(2))
            a = c(3)
            phirad = c(4)*pi/180d0 ! Friction angle in radians
            phirad = atan(tan(phirad)/Msf)
            psirad = c(5)*pi/180d0 ! Dilation angle in radians
            if (psirad.gt.phirad) then  ! Necessary for Safety-calculations
                psirad = phirad
            end if
            sinpsi0 = sin(psirad)
            H0 = c(6)
            e0 = c(7)
            elam = c(8)
            xlams = c(9)
            e = (e0+epsV)/(1-epsV)
            pref = 1d1**((elam-e)/xlams)

            do i =1,6
                    do j = 1,6
                        Dm(i,j) =0d0
                    end do
            end do
            do i =1,3
                    do j = 1,3
                        Dm(i,j) = (BFac - 2d0/3d0)*xG
                    end do
                    Dm(i,i) = (BFac + 4d0/3d0)*xG
                    Dm (i+3,i+3) = xG

            end do
            D = Dm

        end subroutine

        subroutine calcstrinv(Si)
            use vars
            Implicit Double Precision (A-H,O-Z)
            real(8) :: Si1, Si2, Si3, xN1(3), xN2(3), xN3(3)
            real(8), dimension(6), intent(in) :: Si
            integer iOpt

            iOpt = 0
            call PrnSig(iOpt,Si,xN1,xN2,xN3,Si1,Si2,Si3,p,q)
            if (p .lt.(-a)) then
                ipls = 2
            end if
            xJ2=((Si1-Si2)**2d0+(Si2-Si3)**2d0+(Si3-Si1)**2d0)*1d0/6d0 !Calculating J2 from principal stresses

        end subroutine

        subroutine calcond(fs)
            use vars
            Implicit Double Precision (A-H,O-Z)
            double precision, intent(out) :: fs
            double precision :: dSigtr(6), f0, beta

            call calcstrinv(v0(1:6))
            f0 = sqrt(xJ2)-alpha*(p)*3d0-xky ! yield criterion using I1 = 3*p

            call MatVec(Dm,6,de,6,dSigtr) ! Elastic stress increment
            call AddVec(v0(1:6),dSigtr,1d0,-1d0,6,Sigtr) ! Trial stress. Minus sign due to sign convention.
            call calcstrinv(Sigtr)
            fs = sqrt(xJ2)-alpha*(p)*3d0-xky ! yield criterion using I1 = 3*p

           if (fs.gt.1d-6) then
                if (f0.lt.-1d-6) then
                    beta = f0/(f0-fs)
                    call MulVec(dSigtr,beta,size(dSigtr))
                    call AddVec(v0(1:6),dSigtr,1d0,-1d0,6,vs0(1:6))
                    vs0(7) = v0(7)
                else
                    call CopyRVec(v0,vs0,7)
                end if
            end if
        end subroutine

        subroutine differentials(A0)
            use vars
            Implicit Double Precision (A-H,O-Z)
            double precision, dimension(6), intent(out) :: A0
            double precision :: sig11,sig22,sig33,sig12,sig23,sig31
            double precision :: t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12
            double precision :: t13,t14,t15,t16,t17,t18,t19,t20,t21,t22
            double precision :: t23,t24,t25,t26,t27,t28,t29,t30,t31

                  call MZeroR(A0,6)
                  sig11 = vs(1)
                  sig22 = vs(2)
                  sig33 = vs(3)
                  sig12 = vs(4)
                  sig23 = vs(5)
                  sig31 = vs(6)
                  xlam = vs(7)

      t2 = sig22*(1.0D0/3.0D0)
      t4 = sig11*(1.0D0/3.0D0)
      t15 = sig33*(2.0D0/3.0D0)
      t3 = t2+t4-t15
      t5 = sig33*(1.0D0/3.0D0)
      t14 = sig22*(2.0D0/3.0D0)
      t6 = t4+t5-t14
      t20 = sig11*(2.0D0/3.0D0)
      t7 = t2+t5-t20
      t8 = sqrt(3.0D0)
      t9 = 1.0D0/pref
      t10 = sinpsi0**2
      t11 = t10+3.0D0
      t12 = 1.0D0/sqrt(t11)
      t13 = t2+t4+t5
      t16 = t3**2
      t17 = t16*(1.0D0/2.0D0)
      t18 = t6**2
      t19 = t18*(1.0D0/2.0D0)
      t21 = t7**2
      t22 = t21*(1.0D0/2.0D0)
      t23 = sig12**2
      t24 = sig23**2
      t25 = sig31**2
      t26 = t17+t19+t22+t23+t24+t25! t26 = xJ2
      if (t26.lt.1d-6) then
        t26 = 1d-6
      end if
      t27 = 1.0D0/sqrt(t26)
      t28 = t9*t13*(1.0D0/2.0D0)
      t29 = t28-1.0D0
      t30 = sinpsi0*t8*t12*t29*(1.0D0/9.0D0)
      t31 = sinpsi0*t8*t9*t12*t13*(1.0D0/1.8D1)
      A0(1) = t30+t31-t7*t27*(1.0D0/2.0D0)
      A0(2) = t30+t31-t6*t27*(1.0D0/2.0D0)
      A0(3) = t30+t31-t3*t27*(1.0D0/2.0D0)
      A0(4) = sig12*t27
      A0(5) = sig23*t27
      A0(6) = sig31*t27


        end subroutine

        subroutine resi(rr)
            use vars
            Implicit Double Precision (A-H,O-Z)
            double precision, dimension(6) :: depspl, diffsigsigtr
            double precision, dimension(6) :: deltasig, dqdsig
            double precision, dimension(7), intent(out) :: rr

            call MZeroR(rr,7)
            call MZeroR(diffsigsigtr,6)
            call MZeroR(dqdsig,6)
            call MZeroR(depspl,6)
            call MZeroR(deltasig,6)

            call AddVec(vs(1:6),Sigtr,1d0,-1d0,6,diffsigsigtr)
            call differentials(dqdsig)
            call MatVec(Dm,6,dqdsig,6,depspl)
            call MulVec(depspl,(vs(7)-v0(7)),6)
            call AddVec(diffsigsigtr,depspl,1d0,1d0,6,rr(1:6))
            call calcstrinv(vs(1:6))
            call alfcalc()
            rr(7) = sqrt(xJ2)-alpha*(p)*3d0-xky ! yield criterion using I1 = 3*p

        end subroutine

        subroutine jacobian(A0)
            use vars
            Implicit Double Precision (A-H,O-Z)
            double precision, dimension(7,7), intent(out) :: A0
            double precision :: sig11,sig22,sig33,sig12,sig23,sig31,xK
            double precision :: xlam,xlam0
            double precision :: t2,t3,t4,t5,t6,t7,t8,t9,t10,t11,t12
            double precision :: t13,t14,t15,t16,t17,t18,t19,t20,t21,t22
            double precision :: t23,t24,t25,t26,t27,t28,t29,t30,t50,t31
            double precision :: t32,t33,t34,t53,t35,t36,t37,t56,t38,t39
            double precision :: t40,t41,t42 t43,t44,t45,t46,t47,t48,t49
            double precision :: t51,t52,t54,t55,t57,t58,t59,t60,t61,t62
            double precision :: t63,t64,t65,t66,t67,t68,t69,t70,t71,t72
            double precision :: t73,t74,t75,t76,t77,t78,t79,t80,t81,t82
            double precision :: t83,t84,t85,t86,t87,t88,t89,t90

      sig11 = vs(1)
      sig22 = vs(2)
      sig33 = vs(3)
      sig12 = vs(4)
      sig23 = vs(5)
      sig31 = vs(6)
      xlam  = abs(vs(7))
      xlam0 = abs(v0(7))
      xK = BFac*xG
      sinphi = sin(phirad)

            t3 = sig11*(1.0D0/3.0D0)
      t5 = sig22*(1.0D0/3.0D0)
      t8 = sig33*(2.0D0/3.0D0)
      t2 = t3+t5-t8
      t6 = sig33*(1.0D0/3.0D0)
      t12 = sig22*(2.0D0/3.0D0)
      t4 = t3+t6-t12
      t9 = sig11*(2.0D0/3.0D0)
      t7 = t5+t6-t9
      t10 = t2**2
      t11 = t10*(1.0D0/2.0D0)
      t13 = t4**2
      t14 = t13*(1.0D0/2.0D0)
      t15 = t7**2
      t16 = t15*(1.0D0/2.0D0)
      t17 = sig12**2
      t18 = sig23**2
      t19 = sig31**2
      t20 = t11+t14+t16+t17+t18+t19! t20 = xJ2
      if (t20.lt.1d-6) then
        t20 = 1d-6
      end if
      t21 = xG*(2.0D0/3.0D0)
      t22 = t21-xK
      t23 = 1.0D0/sqrt(t20)
      t24 = t23*(1.0D0/6.0D0)
      t25 = 1.0D0/t20**(3.0D0/2.0D0)
      t26 = sqrt(3.0D0)
      t27 = 1.0D0/pref
      t28 = sinpsi0**2
      t29 = t28+3.0D0
      t30 = 1.0D0/sqrt(t29)
      t31 = xlam-xlam0
      t32 = t23*(1.0D0/3.0D0)
      t33 = sinpsi0*t26*t27*t30*(1.0D0/2.7D1)
      t34 = xG*(4.0D0/3.0D0)
      t35 = t34+xK
      t36 = t4*t7*t25*(1.0D0/4.0D0)
      t37 = t2*t7*t25*(1.0D0/4.0D0)
      t38 = t2*t4*t25*(1.0D0/4.0D0)
      t39 = t24-t33+t38
      t40 = t3+t5+t6
      t41 = t27*t40*(1.0D0/2.0D0)
      t42 = t41-1.0D0
      t43 = sinpsi0*t26*t30*t42*(1.0D0/9.0D0)
      t44 = sinpsi0*t26*t27*t30*t40*(1.0D0/1.8D1)
      t58 = t15*t25*(1.0D0/4.0D0)
      t45 = t32+t33-t58
      t46 = t24-t33+t36
      t47 = t35*t46
      t48 = t24-t33+t37
      t62 = t13*t25*(1.0D0/4.0D0)
      t49 = t32+t33-t62
      t66 = t10*t25*(1.0D0/4.0D0)
      t50 = t32+t33-t66
      t51 = t22*t50
      t52 = sig12*t2*t22*t25*(1.0D0/2.0D0)
      t53 = sig23*t2*t22*t25*(1.0D0/2.0D0)
      t54 = sig31*t2*t22*t25*(1.0D0/2.0D0)
      t74 = t4*t23*(1.0D0/2.0D0)
      t55 = t43+t44-t74
      t73 = t2*t23*(1.0D0/2.0D0)
      t56 = t43+t44-t73
      t75 = t7*t23*(1.0D0/2.0D0)
      t57 = t43+t44-t75
      t59 = t22*t45
      t60 = t35*t48
      t61 = t22*t46
      t63 = t22*t49
      t64 = t35*t39
      t65 = t22*t39
      t67 = sig12*t4*t22*t25*(1.0D0/2.0D0)
      t68 = sig12*t7*t22*t25*(1.0D0/2.0D0)
      t69 = sig23*t4*t22*t25*(1.0D0/2.0D0)
      t70 = sig23*t7*t22*t25*(1.0D0/2.0D0)
      t71 = sig31*t4*t22*t25*(1.0D0/2.0D0)
      t72 = sig31*t7*t22*t25*(1.0D0/2.0D0)
      t76 = t23*t31*xG
      t77 = 1.0D0/sin(phirad)
      t78 = H0*t77*xlam
      t79 = t78+1.0D0
      t80 = 1.0D0/t79
      t81 = H0**2
      t82 = xlam**2
      t83 = 1.0D0/t79**2
      t84 = t81*t82*t83
      t85 = t84+3.0D0
      t86 = 1.0D0/sqrt(t85)
      t87 = 1.0D0/t85**(3.0D0/2.0D0)
      t88 = t81*t83*xlam*2.0D0
      t89 = 1.0D0/t79**3
      t90 = t88-H0*t77*t81*t82*t89*2.0D0
      A0(1,1) = t31*(t35*t45+t22*(t24+t36-sinpsi0*t26*t27*t30*(1.0D0/2.7
     +D1))+t22*(t24+t37-sinpsi0*t26*t27*t30*(1.0D0/2.7D1)))+1.0D0
      A0(1,2) = -t31*(t47+t63-t22*t39)
      A0(1,3) = -t31*(t51+t60-t22*t39)
      A0(1,4) = -t31*(t52+t67-sig12*t7*t25*t35*(1.0D0/2.0D0))
      A0(1,5) = -t31*(t53+t69-sig23*t7*t25*t35*(1.0D0/2.0D0))
      A0(1,6) = -t31*(t54+t71-sig31*t7*t25*t35*(1.0D0/2.0D0))
      A0(1,7) = -t22*t55-t22*t56+t35*t57
      A0(2,1) = -t31*(t47+t59-t22*t48)
      A0(2,2) = t31*(t61+t65+t35*t49)+1.0D0
      A0(2,3) = -t31*(t51+t64-t22*t48)
      A0(2,4) = -t31*(t52+t68-sig12*t4*t25*t35*(1.0D0/2.0D0))
      A0(2,5) = -t31*(t53+t70-sig23*t4*t25*t35*(1.0D0/2.0D0))
      A0(2,6) = -t31*(t54+t72-sig31*t4*t25*t35*(1.0D0/2.0D0))
      A0(2,7) = -t22*t56-t22*t57+t35*t55
      A0(3,1) = -t31*(t59+t60-t61)
      A0(3,2) = -t31*(-t61+t63+t64)
      A0(3,3) = t31*(t65+t22*t48+t35*t50)+1.0D0
      A0(3,4) = -t31*(t67+t68-sig12*t2*t25*t35*(1.0D0/2.0D0))
      A0(3,5) = -t31*(t69+t70-sig23*t2*t25*t35*(1.0D0/2.0D0))
      A0(3,6) = -t31*(t71+t72-sig31*t2*t25*t35*(1.0D0/2.0D0))
      A0(3,7) = -t22*t55-t22*t57+t35*t56
      A0(4,1) = sig12*t7*t25*t31*xG*(1.0D0/2.0D0)
      A0(4,2) = sig12*t4*t25*t31*xG*(1.0D0/2.0D0)
      A0(4,3) = sig12*t2*t25*t31*xG*(1.0D0/2.0D0)
      A0(4,4) = t76-t17*t25*t31*xG+1.0D0
      A0(4,5) = -sig12*sig23*t25*t31*xG
      A0(4,6) = -sig12*sig31*t25*t31*xG
      A0(4,7) = sig12*t23*xG
      A0(5,1) = sig23*t7*t25*t31*xG*(1.0D0/2.0D0)
      A0(5,2) = sig23*t4*t25*t31*xG*(1.0D0/2.0D0)
      A0(5,3) = sig23*t2*t25*t31*xG*(1.0D0/2.0D0)
      A0(5,4) = -sig12*sig23*t25*t31*xG
      A0(5,5) = t76-t18*t25*t31*xG+1.0D0
      A0(5,6) = -sig23*sig31*t25*t31*xG
      A0(5,7) = sig23*t23*xG
      A0(6,1) = sig31*t7*t25*t31*xG*(1.0D0/2.0D0)
      A0(6,2) = sig31*t4*t25*t31*xG*(1.0D0/2.0D0)
      A0(6,3) = sig31*t2*t25*t31*xG*(1.0D0/2.0D0)
      A0(6,4) = -sig12*sig31*t25*t31*xG
      A0(6,5) = -sig23*sig31*t25*t31*xG
      A0(6,6) = t76-t19*t25*t31*xG+1.0D0
      A0(6,7) = sig31*t23*xG
      A0(7,1) = -t75-H0*t26*t80*t86*xlam*(1.0D0/3.0D0)
      A0(7,2) = -t74-H0*t26*t80*t86*xlam*(1.0D0/3.0D0)
      A0(7,3) = -t73-H0*t26*t80*t86*xlam*(1.0D0/3.0D0)
      A0(7,4) = sig12*t23
      A0(7,5) = sig23*t23
      A0(7,6) = sig31*t23
      A0(7,7) = -H0*a*t26*t80*t86-H0*t26*t40*t80*t86+H0*a*t26*t80*t87*t9
     +0*xlam*(1.0D0/2.0D0)+H0*t26*t40*t80*t87*t90*xlam*(1.0D0/2.0D0)+a*t
     +26*t77*t81*t83*t86*xlam+t26*t40*t77*t81*t83*t86*xlam



        end subroutine
