      Subroutine User_Mod ( IDTask, iMod, IsUndr,
     *                      iStep, iTer, iEl, Int,
     *                      X, Y, Z,
     *                      Time0, dTime,
     *                      Props, Sig0, Swp0, StVar0,
     *                      dEps, D, BulkW,
     *                      Sig, Swp, StVar, ipl,
     *                      nStat, NonSym, iStrsDep, iTimeDep,iTang,
     *                      iPrjDir, iPrjLen, iAbort )
!
!  Depending on IDTask, 1 : Initialize state variables
!                       2 : calculate stresses,
!                       3 : calculate material stiffness matrix
!                       4 : return number of state variables
!                       5 : inquire matrix properties
!                       6 : calculate elastic material stiffness matrix
!                           return switch for non-symmetric D-matrix
!                           stress/time dependent matrix
! Arguments:
!          I/O  Type
!  IDTask   I   I    : see above
!  iMod     I   I    : model number (1d0.10)
!  IsUndr   I   I    : =1 for undrained, 0 otherwise
!  iStep    I   I    : Global step number
!  iter     I   I    : Global iteration number
!  iel      I   I    : Global element number
!  Int      I   I    : Global integration point number
!  X        I   R    : X-Position of integration point
!  Y        I   R    : Y-Position of integration point
!  Z        I   R    : Z-Position of integration point
!  Time0    I   R    : Time at start of step
!  dTime    I   R    : Time increment
!  Props    I   R()  : List with model parameters
!  Sig0     I   R()  : Stresses at start of step
!  Swp0     I   R    : Excess pore pressure start of step
!  StVar0   I   R()  : State variable at start of step
!  dEps     I   R()  : Strain increment
!  D       I/O  R(,) : Material stiffness matrix
!  BulkW   I/O  R    : Bulkmodulus for water (undrained only)
!  Sig      O   R()  : Resulting stresses
!  Swp      O   R    : Resulting excess pore pressure
!  StVar    O   R()  : Resulting values state variables
!  ipl      O   I    : Plasticity indicator
!  nStat    O   I    : Number of state variables
!  NonSym   O   I    : Non-Symmetric D-matrix ?
!  iStrsDep O   I    : =1 for stress dependent D-matrix
!  iTimeDep O   I    : =1 for time dependent D-matrix
!  iAbort   O   I    : =1 to force stopping of calculation
!


! Contents of Props:
!  1 : xG      Shear modulus
!  2 : xNu    Poissons' ratio
!  3 : Phi    Friction angle
!  4 : Coh    Cohesion
!  5 : Psi    Dilation angle
!  6 : B      Initial steepness controlling plastic stiffness


        use vars
        Implicit Double Precision (A-H,O-Z)

        Integer :: IDTask, iMod, IsUndr, iStep, iter, iel, Int
        Real(8) :: X,Y,Z,Time0,dTime,Props(50),Sig0(20),
     &			       Swp0, StVar0(nStat), dEps(12)
     	Real(8) :: D(6,6), BulkW, diffvec(7)
     	Real(8) :: Sig(6),Swp,StVar(nStat)
     	Integer :: NonSym,iStrsDep,iTimeDep,iTang,iAbort
     	Integer :: nStat
      	Integer :: iPrjDir(iPrjLen),iPrjLen, k

        integer :: stats                                            !Hjelpevariabel. Ikke inkluder i masteroppgave

        open(unit=33, file='c:\\fortran\\errs.txt',                 !Hjelpefunksjon. Ikke inkluder i masteroppgave
     &      status = "replace",                                     !Hjelpefunksjon. Ikke inkluder i masteroppgave
     &          action="write",position="append", iostat=stats)     !Hjelpefunksjon. Ikke inkluder i masteroppgave


        if(IDTask.eq.1) then
            call CopyRVec(Props,c,size(c))
            call MZeroR(StVar0,7)
            call MZeroR(v0,7)
            call MZeroR(v,7)
            nrabort = 1
        end if  !End IDTask 1

        if(IDTask.eq.2) then


        ipls = 0 ! Resetting tension point marker

        ! Activate internal parameters and start values
        if(ALL(StVar0(1:6).eq.0d0)) then
            call CopyRVec(-Sig0,StVar0(1:6),6)
            call CopyRVec(StVar0,v0,7)
        else
            call CopyRVec(StVar0,v0,7)
        end if

        ! Activate strains
        call CopyRVec(dEps(1:6),de,6)
        epsV = dEps(1) + dEps(2) + dEps(3) + dEps(7) + dEps(8) + dEps(9)

        ! Calculate trial stresses and yield condition
        vs(7) = v0(7)
        call calcond(f)

        ! Find correct stress, Sig
        if (ipls.eq.2) then !Apex point. ipls marker set in last subroutine
            v = (/-a,-a,-a,0d0,0d0,0d0,StVar0(7)/) ! "Calculating" v
            call CopyRVec(v,StVar,7) ! Storing v in StVar
            call MulVec(v,-1d0,7) ! Due to sign convention
            call CopyRVec(v(1:6),Sig,6) ! Stress = initial stress (tension cut-off)
            ipl = 2 ! Tension cut-off point
        elseif(f.ge.1d-6) then ! Plastic increment
            call NR(v) ! Calculating v
            nrabort = 1
            call CopyRVec(v,StVar,7) ! Storing v in StVar
            call MulVec(v,-1d0,7) ! Due to sign convention
            call CopyRVec(v(1:6),Sig,6) ! Stress = NR-solution
            ipl = 1 ! Failure
        else !Elastic increment
            call CopyRVec(Sigtr,v(1:6),6) ! "Calculating" v
            v(7) = StVar0(7) ! No change
            call CopyRVec(v,StVar,7) ! Storing v in StVar
            call MulVec(Sigtr,-1d0,6) ! Due to sign convention
            call CopyRVec(Sigtr,Sig,6) ! Stress = trial stress
            ipl = 0 ! Elastic
        end if

        if (IsUndr.eq.1) then
            Swp = Swp0 + BulkW*(dEps(1)+dEps(2)+dEps(3)) ! delta u = K_w*eps_V
        else
            Swp = Swp0
        end if

        end if !End IDTask 2


        if(IDTask.eq.3) then
        if (Sig0(15).gt.1d-2)then     ! Avoiding dividing by zero
            Msf = Sig0(15)
        end if
        call Dmatrix(D)
        end if  !End IDTask 3

        if(IDTask.eq.4) then
        nStat = size(v)
        end if !End IDTask 4

        if(IDTask.eq.5) then
        NonSym = 0 ! 1 for non-symmetric D-matrix
        iStrsDep = 1 ! 1 for stress dependent D-matrix
        iTang = 1 ! 1 for tangent D-matrix
        iTimeDep = 0 ! 1 for time dependent D-matrix
        end if !End IDTask 5

        if(IDTask.eq.6) then
        if (Sig0(15).gt.1d-2)then      ! Avoiding dividing by zero
            Msf = Sig0(15)
        end if
        call Dmatrix(D)
        end if !End IDTask 6

        end subroutine


        include 'vars.f'
        include 'helper.f'
        include 'drupra.f'
        include 'usr_add.f' ! give parameter names and units
        include 'usr_lib.f'
