! Subroutines in this file:
! Exported:
!  Subroutine GetModelCount( nMod )
!  Subroutine GetModelName ( iMod , ModelName )
!  Subroutine GetParamCount( iMod , nParam )
!  Subroutine GetParamName ( iMod , iParam, ParamName )
!  Subroutine GetParamUnit ( iMod , iParam, Units )
!
! Local:
!  Subroutine GetParamAndUnit( iMod , iParam, ParamName, Units )
!


      Subroutine GetModelCount(nMod)
      !
      ! Return the maximum model number (iMod) in this DLL
      !
      Integer (Kind=4) nMod

      nMod = 1 ! Maximum model number (iMod) in current DLL

      Return
      End ! GetModelCount

      Subroutine GetModelName( iMod , ModelName )
      !
      ! Return the name of the different models
      !
      Integer  iMod
      Character (Len= * ) ModelName
      Character (Len=255) tName

      Select Case (iMod)
        Case (1)
          tName = 'Drucker-Prager'
        Case Default
          tName = 'not in DLL'
      End Select
      LT = Len_Trim(tName)
      ModelName= Char(lt) // tName(1:Lt)
      Return
      End ! GetModelName

      Subroutine GetParamCount( iMod , nParam )
      !
      ! Return the number of parameters of the different models
      !
      Select Case (iMod)
        Case ( 1 )
          nParam = 10
        Case Default
          nParam = 0
      End Select
      Return
      End ! GetParamCount

      Subroutine GetParamName( iMod , iParam, ParamName )
      !
      ! Return the parameters name of the different models
      !
      Character (Len=255) ParamName, Units
      Call GetParamAndUnit(iMod,iParam,ParamName,Units)
      Return
      End

      Subroutine GetParamUnit( iMod , iParam, Units )
      !
      ! Return the units of the different parameters of the different models
      !
      Character (Len=255) ParamName, Units
      Call GetParamAndUnit(iMod,iParam,ParamName,Units)
      Return
      End

      Subroutine GetParamAndUnit( iMod , iParam, ParamName, Units )
      !
      ! Return the parameters name and units of the different models
      !
      ! Units: use F for force unit
      !            L for length unit
      !            T for time unit
      !
      Character (Len=255) ParamName, Units, tName
      Select Case (iMod)
        Case (1)
          ! ModName = 'DP'
          Select Case (iParam)
            Case (1)
              ParamName = 'G'     ; Units     = 'F/L^2#'
            Case (2)
              ParamName = '@n#'      ; Units     = '-'
            Case (3)
              ParamName = 'a'  ; Units     = 'F/L^2#'
            Case (4)
              ParamName = '@f#'   ; Units     = 'deg'
            Case (5)
              ParamName = '@y#'   ; Units     = 'deg'
            Case (6)
              ParamName = 'H_0#'   ; Units     = '-'
            Case (7)
              ParamName = 'e_0#'   ; Units     = '-'
            Case (8)
              ParamName = 'e_@l##'   ; Units     = '-'
            Case (9)
              ParamName = '@l#*'   ; Units     = '-'
            Case (10)
              ParamName = 'p*'   ; Units     = 'F/L^2#'
            Case Default
              ParamName = '???'     ; Units     = '???'
          End Select
        Case Default
          ! model not in DLL
          ParamName = ' N/A '        ; Units     = ' N/A '
      End Select
      tName    = ParamName
      LT       = Len_Trim(tName)
      ParamName= Char(lt) // tName(1:Lt)

      tName = Units
      LT    = Len_Trim(tName)
      Units = Char(lt) // tName(1:Lt)

      Return
      End ! GetParamAndUnit


